#include <likelihood.h>
#include "qubxhjc.h"
// for console: #include "asmffi.h"

using DCProgs::t_Bursts;
using DCProgs::t_Burst;
using DCProgs::Log10Likelihood;
using DCProgs::QMatrix;

extern "C" void* QUBXHJC_API qub_hjc_bursts_alloc()
{
  return new t_Bursts;
}

extern "C" void QUBXHJC_API qub_hjc_bursts_free(void *obj)
{
  delete (t_Bursts*) obj;
}

extern "C" void QUBXHJC_API qub_hjc_bursts_append(void *obj, float *durations_ms, int count)
// assuming alternation open,closed,open,...,closed,open; count is odd
// and tDead already applied
{
  t_Bursts *bursts = (t_Bursts*) obj;
  bursts->push_back(t_Burst());
  t_Burst& burst = (*bursts)[bursts->size()-1];
  burst.resize(count);

  //console << "burst:" << std::endl;
  for (int i=0; i<count; ++i) {
    burst[i] = 1e-3 * durations_ms[i];
    //console << durations_ms[i] << std::endl;
  }
}

extern "C" double QUBXHJC_API qub_hjc_likelihood(void *_bursts, double *Q, int numStates, int numOpen, double tDead_ms, double tCrit_ms, int useCHSvectors)
{
  t_Bursts *bursts = (t_Bursts*) _bursts;

  QMatrix qmatrix;
  qmatrix.matrix.resize(numStates, numStates);
  int iq = 0;
  //console << "Q:" << std::endl;
  for (int i=0; i<numStates; ++i) {
    for (int j=0; j<numStates; ++j, ++iq) {
      qmatrix.matrix(i,j) = Q[iq];
      //console << Q[iq] << '\t';
    }
    //console << std::endl;
  }
  qmatrix.nopen = numOpen;

  //console << "nopen: " << qmatrix.nopen << ", tDead: " << tDead_ms << ", tCrit: " << tCrit_ms << ", chs: " << useCHSvectors << std::endl;
  Log10Likelihood likelihood(*bursts, qmatrix.nopen, 1e-3*tDead_ms, useCHSvectors ? (1e-3*tCrit_ms) : -1.0);
  return likelihood(qmatrix);
}

